#include <algorithm>
#include <iostream>
#include <string>
#include <vector>

// [l, r)
using QueryRange = std::pair<int32_t, int32_t>;

// Use a dynamic vector for the children, and iterate over all children to find
// next node. Uses much less memory than a hardcoded array.
class Trie {
public:
    Trie(): children_(1), wordCount_(1, 0) {}

    uint32_t follow(uint32_t node, uint32_t x) const {
        for (auto [val, idx] : children_[node]) {
            if (val == x) {
                return idx;
            }
        }

        return 0;
    }

    uint32_t create(uint32_t node, uint32_t x) {
        uint32_t nextNode = children_.size();
        children_[node].emplace_back(x, nextNode);
        children_.emplace_back();

        wordCount_.push_back(0);

        return nextNode;
    }

    void insert(std::string_view data) {
        uint32_t node = 0;
        for (char c : data) {
            uint32_t x = c - BASE;
            uint32_t nextNode = follow(node, x);
            if (!nextNode) {
                nextNode = create(node, x);
            }

            node = nextNode;
        }

        wordCount_[node]++;
    }

    QueryRange getRange(std::string_view data) const {
        uint32_t node = 0;
        for (char c : data) {
            uint32_t x = c - BASE;
            uint32_t nextNode = follow(node, x);
            if (!nextNode) {
                return EMPTY;
            }

            node = nextNode;
        }

        return ranges_[node];
    }

    // Once all strings are inserted, assign all leaves IDs from 0..k
    // in lexicographic order. Cache on each node the ID range.
    void finalize() {
        ranges_.assign(children_.size(), EMPTY);
        uint32_t id = 0;
        finalize(0, id);
    }

private:
    static constexpr char BASE = 'a';
    static constexpr QueryRange EMPTY{1 << 30, -1};

    QueryRange finalize(uint32_t node, uint32_t& nextId) {
        std::sort(children_[node].begin(), children_[node].end());
        if (wordCount_[node] > 0) {
            ranges_[node].first = nextId;
            ranges_[node].second = nextId + wordCount_[node];
            nextId = ranges_[node].second;
        }

        for (auto [edge, nextNode] : children_[node]) {
            QueryRange sub = finalize(nextNode, nextId);
            ranges_[node].first  = std::min(ranges_[node].first, sub.first);
            ranges_[node].second = std::max(ranges_[node].first, sub.second);
            nextId = ranges_[node].second;
        }

        return ranges_[node];
    }

    // edge label, next node
    std::vector<std::vector<std::pair<uint32_t, uint32_t>>> children_;
    std::vector<QueryRange> ranges_;
    std::vector<int32_t> wordCount_;
};

struct MergeSortTree {
    MergeSortTree(int32_t n, const std::vector<std::pair<int32_t, int32_t>>& points): n_(n), data_(2 * n) {
        for (auto [x, y] : points) {
            data_[x + n_].push_back(y);
        }

        for (int32_t i = n_; i < data_.size(); ++i) {
            std::sort(data_[i].begin(), data_[i].end());
        }

        for (int32_t i = n_ - 1; i > 0; --i) {
            std::merge(
                data_[2 * i].begin(), data_[2 * i].end(),
                data_[2 * i + 1].begin(), data_[2 * i + 1].end(),
                std::back_inserter(data_[i])
            );
        }
    }

    // Count the points <= y in [x0, x1)
    int32_t query(const QueryRange& xs, int32_t y) const {
        int32_t ans = 0;
        auto [l, r] = xs;
        for (l += n_, r += n_; l < r; l /= 2, r /= 2) {
            if (l & 1) {
                ans += std::distance(data_[l].begin(), std::upper_bound(data_[l].begin(), data_[l].end(), y));
                ++l;
            }
            if (r & 1) {
                --r;
                ans += std::distance(data_[r].begin(), std::upper_bound(data_[r].begin(), data_[r].end(), y));
            }
        }

        return ans;
    }

    // Count the points in [x0, x1) x [y0, y1)
    int32_t query(const QueryRange& xs, const QueryRange& ys) const {
        return query(xs, ys.second - 1) - query(xs, ys.first - 1);
    }

    int32_t n_;
    std::vector<std::vector<int32_t>> data_;
};

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);

    // given a dictionary and queries
    // map all words in one set, all reversed words in another set
    // Then we need to do efficient 2d queries with additions offline.

    int32_t n, q;
    std::cin >> n >> q;
    std::vector<std::string> words(n);

    Trie prefix, suffix;
    for (auto& word : words) {
        std::cin >> word;
        prefix.insert(word);

        std::string rev = word;
        std::reverse(rev.begin(), rev.end());
        suffix.insert(rev);
    }
    prefix.finalize();
    suffix.finalize();

    using Point = std::pair<int32_t, int32_t>;
    std::vector<Point> points;
    points.reserve(points.size());
    for (const std::string& word : words) {
        QueryRange prefRange = prefix.getRange(word);

        std::string rev = word;
        std::reverse(rev.begin(), rev.end());

        QueryRange suffRange = suffix.getRange(rev);

        points.emplace_back(prefRange.first, suffRange.first);
    }

    MergeSortTree mergeSortTree(n, points);

    for (int32_t i = 0; i < q; ++i) {
        std::string op, pref, suff;
        std::cin >> op >> pref >> suff;

        std::reverse(suff.begin(), suff.end());

        QueryRange prefRange = prefix.getRange(pref);
        QueryRange suffRange = suffix.getRange(suff);

        int32_t overlap;
        if (prefRange.first >= prefRange.second || suffRange.first >= suffRange.second) {
            overlap = 0;
        } else {
            overlap = mergeSortTree.query(prefRange, suffRange);
        }

        int32_t base = std::max(0, prefRange.second - prefRange.first) + std::max(0, suffRange.second - suffRange.first);

        int32_t ans;
        if (op == "AND") {
            ans = overlap;
        } else if (op == "OR") {
            ans = base - overlap;
        } else if (op == "XOR") {
            ans = base - 2 * overlap;
        } else {
            exit(-1);
        }

        std::cout << ans << '\n';
    }

    return 0;
}
